import os
from cryptography.exceptions import InvalidSignature
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import ec, utils
from cryptography.hazmat.primitives.asymmetric.utils import decode_dss_signature, encode_dss_signature

def generate_identity_keys():
    """
    Generate an ECDSA (P-256) identity key pair.
    Return (private_key, public_key_bytes).
    public_key_bytes is raw x||y each 32 bytes (uncompressed minus the 0x04 prefix).
    """
    private_key = ec.generate_private_key(ec.SECP256R1())
    public_numbers = private_key.public_key().public_numbers()

    x_bytes = public_numbers.x.to_bytes(32, byteorder='big')
    y_bytes = public_numbers.y.to_bytes(32, byteorder='big')
    pubkey_bytes = x_bytes + y_bytes  # 64 bytes

    return private_key, pubkey_bytes


def load_peer_identity_key(pubkey_bytes: bytes):
    """
    Given 64 bytes (x||y) for P-256, return a cryptography public key object.
    """
    if len(pubkey_bytes) != 64:
        raise ValueError("Peer identity pubkey must be exactly 64 bytes (x||y).")
    x_int = int.from_bytes(pubkey_bytes[:32], byteorder='big')
    y_int = int.from_bytes(pubkey_bytes[32:], byteorder='big')
    public_numbers = ec.EllipticCurvePublicNumbers(x_int, y_int, ec.SECP256R1())
    return public_numbers.public_key()


def sign_data(private_key, data: bytes) -> bytes:
    """
    Sign 'data' with ECDSA using P-256 private key.
    Returns DER-encoded signature (variable length, up to ~70-72 bytes).
    """
    signature = private_key.sign(data, ec.ECDSA(hashes.SHA256()))
    return signature


def verify_signature(public_key, signature: bytes, data: bytes) -> bool:
    """
    Verify DER-encoded ECDSA signature with the given public key.
    Return True if valid, False otherwise.
    """
    try:
        public_key.verify(signature, data, ec.ECDSA(hashes.SHA256()))
        return True
    except InvalidSignature:
        return False


def get_ephemeral_keypair():
    """
    Generate ephemeral ECDH keypair (P-256).
    Return (private_key, pubkey_bytes).
    """
    private_key = ec.generate_private_key(ec.SECP256R1())
    numbers = private_key.public_key().public_numbers()
    x_bytes = numbers.x.to_bytes(32, 'big')
    y_bytes = numbers.y.to_bytes(32, 'big')
    return private_key, x_bytes + y_bytes  # 64 bytes


def compute_ecdh_shared_key(private_key, peer_pubkey_bytes: bytes) -> bytes:
    """
    Given a local ECDH private_key and the peer's ephemeral pubkey (64 bytes),
    compute the shared secret.
    """
    x_int = int.from_bytes(peer_pubkey_bytes[:32], 'big')
    y_int = int.from_bytes(peer_pubkey_bytes[32:], 'big')
    peer_public_numbers = ec.EllipticCurvePublicNumbers(x_int, y_int, ec.SECP256R1())
    peer_public_key = peer_public_numbers.public_key()
    shared_key = private_key.exchange(ec.ECDH(), peer_public_key)
    return shared_key


def der_to_raw(der_sig: bytes) -> bytes:
    """
    Convert a DER-encoded ECDSA signature to a raw 64-byte signature (r||s),
    where each component is padded to 32 bytes.
    """
    r, s = decode_dss_signature(der_sig)
    r_bytes = r.to_bytes(32, byteorder='big')
    s_bytes = s.to_bytes(32, byteorder='big')
    return r_bytes + s_bytes


def raw_signature_to_der(raw_sig: bytes) -> bytes:
    """
    Convert a raw signature (64 bytes, concatenated r||s) to DER-encoded signature.
    """
    if len(raw_sig) != 64:
        raise ValueError("Raw signature must be 64 bytes (r||s).")
    from cryptography.hazmat.primitives.asymmetric.utils import encode_dss_signature
    r = int.from_bytes(raw_sig[:32], 'big')
    s = int.from_bytes(raw_sig[32:], 'big')
    return encode_dss_signature(r, s)
